module HaskellUnit where
import Control.Exception
import Data.Maybe
import Data.List
import System.IO

esc '"' = "&quot;"
esc '\'' = "&apos;"
esc '&' = "&amp;"
esc '<' = "&lt;"
esc '>' = "&gt;"
esc c = [c]

xmlEsc = concat.map esc


runTests tests = do
  putStrLn "<?xml version=\"1.0\" ?>"
  putStrLn "<testsuite>"
  xs <- sequence tests
  putStrLn ("  <results errors=\""++show (length (filter not xs))++"\" tests=\""++show (length xs)++"\" failures=\"0\"/>")
  putStrLn "</testsuite>"


compareResult expected actual = do
  catch 
    (if (expected == actual)
      then return Nothing
      else return (Just (xmlEsc ("expected:&lt;"++show expected++"&gt; but was:&lt;"++show actual++"&gt;"))))
    (\e->return (Just (replace "\n" " "("Exception in evaluation: "++ (show (e :: SomeException)))))) 

replace [] _ _ = error "Extra.replace, first argument cannot be empty"
replace from to xs | Just xs <- stripPrefix from xs = to ++ replace from to xs
replace from to (x:xs) = x : replace from to xs
replace from to [] = []


testcase name msg expected actual  = do
    res <- compareResult expected actual
    maybe
      (do
          putStrLn ("  <testcase name=\""++name++"\">")
          hFlush stdout
          putStrLn "  </testcase>"
          hFlush stdout
          return True)
      (\failMessage -> do
          putStrLn ("  <testcase name=\""++name++"\">")
          --let failMessage = "expected:&lt;"++show expected++"&gt; but was:&lt;"++show actual++"&gt;"
          putStrLn ("    <failure message=\""++failMessage++"\" type=\"AssertionFailedError\">")
          putStrLn (msg++": "++failMessage)
          putStrLn "    </failure>"
          putStrLn "  </testcase>"
          return False)
      res

iotestcase name msg expected actualIO  = do
  actual <- actualIO
  testcase name msg expected actual
